﻿local NekoLoot = LibStub("AceAddon-3.0"):GetAddon("NekoLoot")
local L = LibStub("AceLocale-3.0"):GetLocale("NekoLoot")

local FreeSlots = GetContainerNumFreeSlots

local lootMode = nil
local errorDialog = nil

local backdrop = {
	bgFile="Interface\\DialogFrame\\UI-DialogBox-Background" ,
	edgeFile="Interface\\DialogFrame\\UI-DialogBox-Border",
	tile = true,
	insets = {left = 11, right = 12, top = 12, bottom = 11},
	tileSize = 32,
	edgeSize = 32,
}

function NekoLoot:ChangeLooting(mode, enable)
	NekoLoot:UnregisterEvent("LOOT_OPENED")
	local text = ""
	-- mode fast und enable, dann fast
	if mode == "fast" and enable then mode, text = "FastLootOpen", "Fast"
	
	-- mode extraFast und enable, dann extraFast und UI_ERROR Handling an
	elseif mode == "extraFast" and enable then
		mode, text = "ExtraFastLootOpen", "ExtraFast"
		NekoLoot:RegisterEvent("UI_ERROR_MESSAGE", "LootOpenOnError")
	
	-- mode extraFast und disable, dann Normal und UI_ERROR Handling aus
	elseif mode == "extraFast" and not enable then
		mode, text = "NormalLootOpen", "Normal"
		NekoLoot:UnregisterEvent("UI_ERROR_MESSAGE", "LootOpenOnError")
	
	-- mode nicht gesetzt oder modus deaktivieren, dann normal
	else mode, text = "NormalLootOpen", "Normal"
	end
	
	-- report changing to user
	if mode ~= self.lootMode and not (self.lootMode == nil and mode == "NormalLootOpen") then
		NekoLoot:Print(L["LootingChanged"] .. text)
	end
	
	-- store mode to check if changed later
	self.lootMode = mode

	-- finaly register loot mode to event - fired when a corpse is looted 
	NekoLoot:RegisterEvent("LOOT_OPENED", mode)
end

-- simple show lootframe
function NekoLoot:NormalLootOpen()
	NekoLoot:Update()
end

-- check if lootframe is needed
function NekoLoot:FastLootOpen()
	local empty, num = 0, GetNumLootItems()
	local modified = IsModifiedClick(AUTOLOOTTOGGLE)
	local default = GetCVar("autoLootDefault") == "1"
	-- autoloot
	-- versuche schneller zu looten
	if (modified or default) and not (modified and default) then
		for index = 1, NUM_BAG_SLOTS do
			empty = empty + FreeSlots(index)
		end
		
		if num <= empty then CloseLoot()
		else self:Update()
		end
	-- no autoloot
	elseif num > 0 then
		self:Update()
		if not self.lootFrame:IsShown() and IsFishingLoot() then
			PlaySound("FISHING REEL IN")
		end
	else
		CloseLoot()
	end
end

-- only close loot frame to speed up auto loot
function NekoLoot:ExtraFastLootOpen()
	CloseLoot();
end

local function ErrorDialogButtonHandler()
	NekoLoot:ChangeLooting("extraFast", false)
	NekoLoot.db.profile.lootMode.extraFast = false
end

-- check given event for inventory is full msg and if create error dialog
-- event			: event to handle
-- msg				: msg of event
function NekoLoot:LootOpenOnError(event, msg, ...)
	if msg == ERR_INV_FULL then
		UIErrorsFrame:Clear()
		-- NekoLoot:Print(RED_FONT_COLOR_CODE.."Inventar voll! Disable ExtraFastLooting or Cleanup your Bags..."..FONT_COLOR_CODE_CLOSE)
		NekoLoot:CreateErrorDialog(
			ERR_INV_FULL .. L["Error. Disable ExtraFastLooting"],
			ErrorDialogButtonHandler)
	end
end

-- creates an error dialog
-- msg				: message to display on the dialog
-- buttonHandler	: function to call when OK-button is klicked
function NekoLoot:CreateErrorDialog(msg, buttonHandler)
	if not self.errorDialog then
		local dialogFrame = CreateFrame("Frame", "NekoLootErrorDialog", UIParent)
		dialogFrame:SetFrameStrata("DIALOG")
		dialogFrame:SetToplevel(true)
		dialogFrame:EnableMouse(true)
		dialogFrame:SetClampedToScreen(true)
		dialogFrame:SetWidth(360)
		dialogFrame:SetHeight(160)
		dialogFrame:SetBackdrop(backdrop)
		dialogFrame:SetPoint("CENTER", 0, 0)
		dialogFrame:Hide()

		header = dialogFrame:CreateTexture(nil, "ARTWORK")
		header:SetTexture("Interface\\DialogFrame\\UI-DialogBox-Header")
		header:SetWidth(256); header:SetHeight(64)
		header:SetPoint("TOP", 0, 12)
		
		title = dialogFrame:CreateFontString("ARTWORK")
		title:SetFontObject("GameFontNormal")
		title:SetPoint("TOP", header, "TOP", 0, -14)
		title:SetText("NekoLoot")

		desc = dialogFrame:CreateFontString("ARTWORK")
		desc:SetFontObject("GameFontHighlight")
		desc:SetJustifyV("TOP")
		desc:SetJustifyH("LEFT")
		desc:SetPoint("TOPLEFT", 18, -32)
		desc:SetPoint("BOTTOMRIGHT", -18, 48)
		desc:SetText(msg)
	
		button = CreateFrame("CheckButton", "NekoLootErrorDialog", dialogFrame, "OptionsButtonTemplate")
		_G[button:GetName() .. "Text"]:SetText("OK")
		
		button:SetScript("OnClick", function(self) 
			dialogFrame:Hide() 
			buttonHandler()
			end)
		button:SetPoint("BOTTOM", 0, 14)
		
		self.errorDialog = dialogFrame
	end
	self.errorDialog:Show()
end