local NekoLoot = LibStub("AceAddon-3.0"):GetAddon("NekoLoot")
local L = LibStub("AceLocale-3.0"):GetLocale("NekoLoot")

local AceConfDia = LibStub("AceConfigDialog-3.0")
local AceConf = LibStub("AceConfig-3.0")

-- local pairs, type, tostring = pairs, type, tostring

local qualityKeys = {}

local anchorKeys = {
	TOPRIGHT = "TOPRIGHT", TOP = "TOP", TOPLEFT = "TOPLEFT",
	RIGHT = "RIGHT", CENTER = "CENTER", LEFT = "LEFT", 
	BOTTOMRIGHT = "BOTTOMRIGHT", BOTTOM = "BOTTOM", BOTTOMLEFT = "BOTTOMLEFT"
}
local anchorPoints = {
	PARENT = "PARENT", BACKGROUND = "BACKGROUND", LOW = "LOW", MEDIUM = "MEDIUM",
	HIGH = "HIGH", DIALOG = "DIALOG", FULLSCREEN = "FULLSCREEN",
	FULLSCREEN_DIALOG = "FULLSCREEN_DIALOG", TOOLTIP = "TOOLTIP"
}

-- get and set magic
local getOpt, setOpt, getColor, setColor
do
	function getOpt(info)
		-- db.profile.GROUP.KEY --> GROUP is penultimate and KEY is ultimate arg of info
		return NekoLoot.db.profile[info[#info-1]][info[#info]]
	end
	
	function setOpt(info, value)
		-- db.profile.GROUP.KEY --> GROUP is penultimate and KEY is ultimate arg of info
		NekoLoot.db.profile[info[#info-1]][info[#info]] = value
		NekoLoot:ApplySettings()
	end
	
	function getColor(info)
		return unpack(getOpt(info))
	end
	
	function setColor(info, r, g, b, a)
		setOpt(info, {r, g, b, a})
	end
end


local options = nil
local function getOptions()
	if not options then
		options = {
			type = "group", childGroups = "tab",
			args = {
				general = {
					type = "group", inline = false,
					name = L["ConfigFrame/General"],
					args = {
						descText = {
							type = "toggle", order = 10, width = "double",
							name = L["ConfigFrame/General/DescText"],
							desc = L["ConfigFrame/General/DescTextDesc"],
							get = getOpt,
							set = setOpt,
						},
						descColor = {
							type = "color", order = 11, width = "double",
							name = L["ConfigFrame/General/DescColor"],
							desc = L["ConfigFrame/General/DescColorDesc"],
							get = getColor,
							set = setColor,
						},
						textWidth = {
							type = "range", order = 12,
							name = L["ConfigFrame/General/TextWidth"],
							desc = L["ConfigFrame/General/TextWidthDesc"],
							min = 50, max = 1000, step = 1,
							get = getOpt,
							set = setOpt,
						},

						headerFontSize = {
							type = "header", order = 20,
							name = L["ConfigFrame/General/HeaderFontSize"],
						},
						descHeight = {
							type = "range", order = 30,
							name = L["ConfigFrame/General/DescHeight"],
							desc = L["ConfigFrame/General/DescHeightDesc"],
							min = 8, max = 18, step = 1,
							get = getOpt,
							set = setOpt,
						},
						qualityHeight = {
							type = "range", order = 31,
							name = L["ConfigFrame/General/QualityHeight"],
							desc = L["ConfigFrame/General/QualityHeightDesc"],
							min = 8, max = 18, step = 1,
							get = getOpt,
							set = setOpt,
						},
						textHeight = {
							type = "range", order = 32,
							name = L["ConfigFrame/General/TextHeight"],
							desc = L["ConfigFrame/General/TextHeightDesc"],
							min = 8, max = 18, step = 1,
							get = getOpt,
							set = setOpt,
						},
						bindHeight = {
							type = "range", order = 33,
							name = L["ConfigFrame/General/BindHeight"],
							desc = L["ConfigFrame/General/BindHeightDesc"],
							min = 8, max = 18, step = 1,
							get = getOpt,
							set = setOpt,
						},

						generalHeader1 = {
							type = "header", order = 50,
							name = "",
						},
						expand = {
							type = "toggle", order = 60, width = "double",
							name = L["ConfigFrame/General/Expand"],
							desc = L["ConfigFrame/General/ExpandDesc"],
							get = getOpt,
							set = setOpt,
						},
						highlight = {
							type = "select", order = 70, style = "dropdown",
							name = L["ConfigFrame/General/Highlight"],
							desc = L["ConfigFrame/General/HighlightDesc"],
							values = qualityKeys,
							get = function(info)
								return NekoLoot.db.profile.general.highlight + 1
							end,
							set = function(info, value) 
								NekoLoot.db.profile.general.highlight = value - 1
							end,
						},
					},
				},
				---- general
				
				frame = {
					type = "group", inline = false,
					name = L["ConfigFrame/Frame"],
					desc = L["ConfigFrame/FrameDesc"],
					args = {
						scale = {
							type = "range", order = 110,
							name = L["ConfigFrame/Frame/Scale"],
							desc = L["ConfigFrame/Frame/ScaleDesc"],
							min = 0.2, max = 2, step = 0.1,
							get = getOpt,
							set = setOpt,
						},
						alpha = {
							type = "range", order = 111,
							name = L["ConfigFrame/Frame/Alpha"],
							desc = L["ConfigFrame/Frame/AlphaDesc"],
							min = 0.1, max = 1, step = 0.1,
							get = getOpt,
							set = setOpt,
						},
						strata = {
							type = "select", order = 112, style = "dropdown",
							name = L["ConfigFrame/Frame/Strata"],
							desc = L["ConfigFrame/Frame/StrataDesc"],
							values = anchorPoints,
							get = getOpt,
							set = setOpt,
						},

						frameHeader1 = {
							type = "header", order = 120,
							name = L["ConfigFrame/Frame/Alignment"],
						},
						parentAnchor = {
							type = "select", order = 130, style = "dropdown",
							name = L["ConfigFrame/Frame/ParentAnchor"],
							desc = L["ConfigFrame/Frame/ParentAnchorDesc"],
							values = anchorKeys,
							get = getOpt,
							set = setOpt,
						},
						unlock = {
							type = "execute", order = 131,
							name = L["ConfigFrame/Frame/ToggleLock"],
							desc = L["ConfigFrame/Frame/ToggleLockDesc"],
							func = function()
								NekoLoot:ToggleLock(true)
							end,
						},
						lock = {
							type = "toggle", order = 132,
							name = L["ConfigFrame/Frame/Lock"],
							desc = L["ConfigFrame/Frame/LockDesc"],
							get = getOpt,
							set = setOpt,
						},
						frameSpace2 = {
							type = "description", order = 140,
							name = "",
						},
						xOffSet = {
							type = "range", order = 150, width = "double",
							name = L["ConfigFrame/Frame/XOffSet"],
							desc = L["ConfigFrame/Frame/XOffSetDesc"],
							min = -1600, max = 1600, step = 1,
							get = getOpt,
							set = setOpt,
						},
						yOffSet = {
							type = "range", order = 151, width = "double",
							name = L["ConfigFrame/Frame/YOffSet"],
							desc = L["ConfigFrame/Frame/YOffSetDesc"],
							min = -1600, max = 1600, step = 1,
							get = getOpt,
							set = setOpt,
						},
						frameSpace3 = {
							type = "description", order = 160,
							name = "",
						},
						anchorPoint = {
							type = "select", order = 170, style = "dropdown",
							name = L["ConfigFrame/Frame/AnchorPoint"],
							desc = L["ConfigFrame/Frame/AnchorPointDesc"],
							values = anchorKeys,
							get = getOpt,
							set = setOpt,
						},
						frameSpace4 = {
							type = "description", order = 171, width = "normal",
							name = "",
						},
						width = {
							type = "range", order = 172, width = "double",
							name = L["ConfigFrame/Frame/Width"],
							desc = L["ConfigFrame/Frame/WidthDesc"],
							min = 50, max = 1000, step = 1,
							get = getOpt,
							set = setOpt,
						},
						
						frameSpace5 = {
							type = "header", order = 180,
							name = L["ConfigFrame/Frame/Colors"],
						},
						backgroundColor = {
							type = "color", order = 190, hasAlpha = true,
							name = L["ConfigFrame/Frame/BackgroundColor"],
							desc = L["ConfigFrame/Frame/BackgroundColorDesc"],
							get = getColor,
							set = setColor,
						},
						borderColor = {
							type = "color", order = 191, hasAlpha = true,
							name = L["ConfigFrame/Frame/BorderColor"],
							desc = L["ConfigFrame/Frame/BorderColorDesc"],
							get = getColor,
							set = setColor,
						},
					},
				},
				---- frame
			
				loot = {
					type = "group", inline = false,
					name = L["ConfigFrame/Loot"],
					desc = L["ConfigFrame/LootDesc"],
					args = {
						height = {
							type = "range", order = 210,
							name = L["ConfigFrame/Loot/Height"],
							desc = L["ConfigFrame/Loot/HeightDesc"],
							min = 10, max = 100, step = 1,
							get = getOpt,
							set = setOpt,
						},
						
						lootSpace = {
							type = "description", order = 219,
							name = "",
						},
						backgroundColor = {
							type = "color", order = 220, hasAlpha = true,
							name = L["ConfigFrame/Loot/BackgroundColor"],
							desc = L["ConfigFrame/Loot/BackgroundColorDesc"],
							get = getColor,
							set = setColor,
						},
						buttonBorderColor = {
							type = "color", order = 221, hasAlpha = true,
							name = L["ConfigFrame/Loot/ButtonBorderColor"],
							desc = L["ConfigFrame/Loot/ButtonBorderColorDesc"],
							get = getColor,
							set = setColor,
						},
						
						lootHeader1 = {
							type = "header", order = 230,
							name = L["ConfigFrame/Loot/Highlight"],
						},
						highlightIcon = {
							type = "toggle", order = 240,
							name = L["ConfigFrame/Loot/HighlightIcon"],
							desc = L["ConfigFrame/Loot/HighlightIconDesc"],
							get = getOpt,
							set = setOpt,
						},
						highlightButton = {
							type = "toggle", order = 241,
							name = L["ConfigFrame/Loot/HighlightButton"],
							desc = L["ConfigFrame/Loot/HighlightButtonDesc"],
							get = getOpt,
							set = setOpt,
						},
						highlightFrame = {
							type = "toggle", order = 242,
							name = L["ConfigFrame/Loot/HighlightFrame"],
							desc = L["ConfigFrame/Loot/HighlightFrameDesc"],
							get = getOpt,
							set = setOpt,
						},
					},
				},
				---- loot

				lootMode = {
					type = "group", inline = false,
					name = L["ConfigFrame/LootMode"],
					desc = L["ConfigFrame/LootModeDesc"],
					args = {
						warning = {
							type = "header", order = 300, --fontSize = "large",
							-- name = "|cffff0000" .. L["ConfigFrame/LootMode/Warning"],
							name = RED_FONT_COLOR_CODE .. L["ConfigFrame/LootMode/Warning"],
						},
						warningText = {
							type = "description", order = 301, fontSize = "medium",
							name = NORMAL_FONT_COLOR_CODE .. L["ConfigFrame/LootMode/WarningText"],
						},
						
						lootModeSpace1 = {
							type = "description", order = 310, fontSize = "large",
							name = " ",
						},
						
						fast = {
							type = "toggle", order = 340, width = "double",
							name = L["ConfigFrame/LootMode/Fast"],
							get = function(info) return NekoLoot.db.profile.lootMode.Fast end,
							set = function (info, value)
								NekoLoot.db.profile.lootMode.extraFast = false
								NekoLoot.db.profile.lootMode.Fast = value
								NekoLoot:ChangeLooting(info[#info], value)
							end,
						},
						fastDesc = {
							type = "description", order = 341, fontSize = "medium",
							name = NORMAL_FONT_COLOR_CODE .. L["ConfigFrame/LootMode/FastDesc"],
						},
						extraFast = {
							type = "toggle", order = 345, width = "double",
							name = L["ConfigFrame/LootMode/ExtraFast"],
							get = function(info) return NekoLoot.db.profile.lootMode.extraFast end,
							set = function (info, value)
								NekoLoot.db.profile.lootMode.Fast = false
								NekoLoot.db.profile.lootMode.extraFast = value
								NekoLoot:ChangeLooting(info[#info], value)
							end,
						},
						extraFastDesc = {
							type = "description", order = 346, fontSize = "medium",
							name = NORMAL_FONT_COLOR_CODE .. L["ConfigFrame/LootMode/ExtraFastDesc"],
						},
					},
				},
				----- lootMode
				
				config = {
					type = "execute", guiHidden = true,
					name = L["ConfigFrame/Config"],
					desc = L["ConfigFrame/ConfigDesc"],
					func = function()
						if AceConfDia.OpenFrames["NekoLoot"] then
							AceConfDia:Close("NekoLoot")
						else
							AceConfDia:Open("NekoLoot")
						end
					end,
				},
				---- config

				enable = {
					type = "execute", guiHidden = true,
					name = L["ConfigFrame/Enable"],
					desc = L["ConfigFrame/EnableDesc"],
					func = function() self:OnEnable(); end,
				},
				---- enable
				
				disable = {
					type = "execute", guiHidden = true,
					name = L["ConfigFrame/Disable"],
					desc = L["ConfigFrame/DisableDesc"],
					func = function() self:OnDisable() end,
				},
				---- disable
			},
		}
	end
	return options
end


function NekoLoot:SetupOptions()
	for index = 0, 7 do
		qualityKeys[ index + 1] = 
			ITEM_QUALITY_COLORS[ index].hex.._G["ITEM_QUALITY"..tostring(index).."_DESC"].."|r"
	end

	-- register so config changes can get
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("NekoLoot", getOptions)

	AceConf:RegisterOptionsTable("NekoLoot", getOptions, {"NekoLoot", "nl"})
	
	AceConfDia:AddToBlizOptions("NekoLoot", "NekoLoot", nil, "general")

	-- insert wowace profile handling
	getOptions().args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	
	AceConfDia:AddToBlizOptions("NekoLoot", L["Frame"], "NekoLoot", "frame")
	AceConfDia:AddToBlizOptions("NekoLoot", L["Loot"], "NekoLoot", "loot")
	AceConfDia:AddToBlizOptions("NekoLoot", L["LootMode"], "NekoLoot", "lootMode")

	AceConfDia:AddToBlizOptions("NekoLoot", "Profiles", "NekoLoot", "profiles")
end